package com.qmth.teachcloud.common.sync;

import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.teachcloud.common.SignatureEntityTest;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

/**
 * 同步云阅卷接口工具类
 *
 * @Date: 2021/10/29.
 */
public class StmmsUtils {

    // 考试类型（逸教云默认是纸笔"SCAN_IMAGE"）
    private static final String SAVE_EXAM_TYPE = "SCAN_IMAGE";
    // 所有请求方法默认为"POST"
    private static final String POST_METHOD = "POST";

    @Autowired
    private CommonCacheService commonCacheService;

    @Autowired
    private DictionaryConfig dictionaryConfig;

    /**
     * 考试创建/更新接口
     *
     * @param code     唯一标识
     * @param name     考试名称
     * @param examTime 考试时间
     * @return
     */
    public Long syncExam(String code, String name, String examTime) {
//        String hostUrl = dictionaryConfig.syncDataDomain().getHostUrl();
//        String examSaveUrl = dictionaryConfig.syncDataDomain().getExamSaveUrl();
//        String postUrl = hostUrl.concat(examSaveUrl);
//        //参数
//        Map<String, String> map = new HashMap<>();
//        map.put("code", code);
//        map.put("name", name);
//        map.put("examTime", examTime);
//        map.put("type", SAVE_EXAM_TYPE);
//
//        String result = HttpKit.sendPost(postUrl, getHeaders(examSaveUrl), map, null, null, null);
//        JSONObject jsonObject = JSONObject.parseObject(result);
//        if (jsonObject.containsKey("id")) {
//            return Long.valueOf(jsonObject.get("id").toString());
//        } else {
//            throw ExceptionResultEnum.ERROR.exception("考试同步失败");
//        }
        return null;
    }

    /**
     * 考生创建/更新接口
     *
     * @param examId      考试ID
     * @param examNumber  准考证号
     * @param studentCode 学号
     * @param name        姓名
     * @param college     学院
     * @param className   班级名称
     * @param teacher     教师姓名
     * @param subjectCode 科目代码
     * @param subjectName 科目名称
     * @param packageCode 签到表编号
     * @param paperType   试卷类型
     * @param examSite    考点
     * @param examRoom    考场
     * @return
     */
    public Long syncStudent(String examId, String examNumber, String studentCode, String name, String college,
                            String className, String teacher, String subjectCode, String subjectName,
                            String packageCode, String paperType, String examSite, String examRoom) {
        String hostUrl = dictionaryConfig.syncDataDomain().getHostUrl();
        String studentSaveUrl = dictionaryConfig.syncDataDomain().getStudentSaveUrl();
        String postUrl = hostUrl.concat(studentSaveUrl);
//        try {
//            //参数
//            Map<String, String> map = new HashMap<>();
//            map.put("examId", examId);
//            map.put("examNumber", examNumber);
//            map.put("studentCode", studentCode);
//            map.put("name", name);
//            map.put("college", StringUtils.isBlank(college) ? "无" : college);
//            map.put("className", StringUtils);
//            map.put("teacher", "无");
//            map.put("subjectCode", examStudent.getPaperNumber()); // 取试卷编号
//            map.put("subjectName", examStudent.getCourseName());
//
//            String result = HttpKit.sendPost(postUrl, getHeaders(examPrintPlan.getSchoolId(), studentSaveUrl), map, null, null, null);
//            JSONObject jsonObject = JSONObject.parseObject(result);
//            if (jsonObject.containsKey("updateTime")) {
//                UpdateWrapper<ExamStudent> updateWrapper = new UpdateWrapper<>();
//                updateWrapper.lambda().set(ExamStudent::getSyncStatus, true).eq(ExamStudent::getId, examStudent.getId());
//                examStudentService.update(updateWrapper);
//            }
//        } catch (Exception e) {
//            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
//        }
        return  null;
    }

    /**
     * http请求头
     *
     * @param url
     * @return
     */
    private Map<String, String> getHeaders(String url) {
        long time = System.currentTimeMillis();
        Map<String, String> header = new HashMap<>();
        header.put(SystemConstant.HEADER_AUTHORIZATION, createSign(time, url));
        header.put(SystemConstant.HEADER_TIME, String.valueOf(time));
        return header;
    }

    /**
     * 签名
     *
     * @param time
     * @param url
     * @return
     */
    private String createSign(long time, String url) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
        if (basicSchool == null) {
            throw ExceptionResultEnum.ERROR.exception("学校不存在");
        }
        if (!basicSchool.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("学校已禁用");
        }

        String signature = SignatureEntityTest.build(SignatureType.SECRET, POST_METHOD, url, time, basicSchool.getAccessKey(), basicSchool.getAccessSecret());
        return signature;
    }

    private String validParam(String value, String defaulValue, boolean require, String name) {
        if (require && StringUtils.isAnyBlank(name, defaulValue)) {
            throw ExceptionResultEnum.ERROR.exception((StringUtils.isBlank(name) ? "" : name) + "值必填");
        }

        return StringUtils.isBlank(value) ? defaulValue : value;
    }
}
